/*
 * Library that contains some basic functions that movelara uses
 * Copyright (C) 2016  b122251
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your optiolen) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "functions.h"

/* Global declarations */
extern char* level; /* Pointer to the level file in memory */
extern unsigned levelsize; /* Length of the level file in memory */
extern char endian; /* Endianness. 0=Big-Endian, 1=Little-Endian */

/*
 * Reads a file into memory.
 * Parameters: 
 *   inputFile=Name of the file (String)
 * Returns a pointer to the file in memory.
 */
char* readFile(char *inputFile){
	FILE *file;
	if (!(file = fopen(inputFile,"rb"))){
		printf("ERROR: %s could not be opened!\n",inputFile);
		exit(-1);
	}
	fseek(file, 0, SEEK_END);
	unsigned fsize=ftell(file);
	char *memfile = malloc(fsize);
	fseek(file, 0, SEEK_SET);
	fread(memfile, 1, fsize, file);
	fclose(file);
	return memfile;
}

/*
 * Writes a file from memory.
 * Parameters: 
 *   outputFile=Name of the file (String)
 *   memfile=File in memory
 *   fsize=The size of the file (in bytes)
 */
void writeFile(char *outputFile, char *memfile, unsigned fsize){
	FILE *File;
	if (!(File = fopen(outputFile,"wb"))){
		printf("ERROR: %s could not be written to!\n",outputFile);
		exit(-1);
	}
	fseek(File, 0, SEEK_SET);
	fwrite(memfile, 1, fsize, File);
	fclose(File);
}

/*
 * Detects the size of a file (in bytes).
 * Parameters: 
 *   inputFile=Name of the file (String)
 * Returns an unsigned 32-bit integer.
 */
unsigned filesize(char *inputFile){
	FILE *File = fopen(inputFile,"rb");
	fseek(File, 0, SEEK_END);
	unsigned fsize=ftell(File);
	fclose(File);
	return fsize;
}

/*
 * Returns a substring of a string.
 * Parameters: 
 *   string=pointer to the original string
 *   offset=character to start at (counted from 0)
 *   length=number of characters to return.
 * Returns a pointer to the substring.
 */
char* substring(char *string, unsigned offset, unsigned length){
	char *string2 = malloc(length+1);
	memcpy(string2, string+offset, length);
	memset(string2+length,'\0',1);
	return string2;
}

/*
 * Returns a reversed version of a 16-bit integer.
 * Parameters: 
 *   var=original short
 * Returns an unsigned short.
 */
unsigned short reverse16(unsigned short var){
	return (((var&0x00FF)<<8)+((var&0xFF00)>>8));
}

/*
 * Returns a reversed version of a 32-bit integer.
 * Parameters: 
 *   var=original integer.
 * Returns an unsigned integer.
 */
unsigned reverse32(unsigned var){
	return (((var&0x000000FF)<<24)+((var&0x0000FF00)<<8)+((var&0x00FF0000)>>8)+((var&0xFF000000)>>24));
}
