/*
 * Program that moves the starting position of a Tomb Raider level
 * Copyright (C) 2016  b122251
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "main.h"
#include "functions.h"
#include "tr1pc.h"
#include "tr1psx.h"
#include "tr1sat.h"
#include "tr2pc.h"
#include "tr2psx.h"
#include "tr3pc.h"

/* Global declarations */
char* level; /* Pointer to the level file in memory */
unsigned levelsize; /* Length of the level file in memory */
char endian; /* Endianness. 0=Big-Endian, 1=Little-Endian */

/*
 * Main function. It detects endianness, then reads 
 * the input given by the user, and starts the corresponding functions.
 * Parameters: 
 *   argc=The number of command-line arguments
 *   argv=Command-line arguments
 */
int main(int argc, char *argv[]){
	char freechar1;
	short freeshort1=1;
	unsigned freeuint1=0;
	
	/* Detects Endianness */
	memcpy(&endian,&freeshort1,1);
	
	/* Checks input, and after that detects file type and starts the corresponding functions */
	if (argc<7){
		displayText();
	} else {
		level=readFile(argv[1]);
		levelsize=filesize(argv[1]);
		if (!strncasecmp(substring(argv[1],strlen(argv[1])-3,3),"sat",3)){
			tr1sat_main(argc, argv);
		}else{
			if (!strncasecmp(substring(argv[1],strlen(argv[1])-3,3),"psx",3)){
				freechar1=2;
				freeuint1=0x56414270;
				if (!endian) freeuint1=reverse32(freeuint1);
				if (!memcmp(level+4,&freeuint1,4)) freechar1=1;
				if (freechar1==1) tr1psx_main(argc, argv);
				if (freechar1==2) tr2psx_main(argc, argv);
			}else{
				memcpy(&freeuint1,level,4);
				if (!endian) freeuint1=reverse32(freeuint1);
				if (freeuint1==32) tr1pc_main(argc, argv);
				if (freeuint1==45) tr2pc_main(argc, argv);
				if (freeuint1==4278714424||freeuint1==4279763000) tr3pc_main(argc, argv);
			}
		}
	}
	return 0;
}

/* Displays the standard message, containing basic syntax and license information */
void displayText(){
	system("clear"); /* Replace with "cls" on Windows */
	printf("Move Lara\n");
	printf("=========\n\n");
	printf("This program can change the starting position of Tomb Raider level files.\n\n");
	printf("movelara [level] [room] [X] [Z] [Y] [Angle]\n\n");
	printf("This program comes with ABSOLUTELY NO WARRANTY.\n");
	printf("This program is distributed under the GNU General Public License 3.0.\n");
	printf("This program is dedicated to Ludko444.\n\n\n");
	printf("MoveLara 1.0 Copyright (C) 2016 b122251\n");
	printf("<b122251.org>\n\n");
}

