/*
 * Functions for Editing Tomb Raider 1 PC Files (PHD and TUB)
 * Copyright (C) 2016  b122251
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your optiolen) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/* File Inclusions */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "functions.h"
#include "tr1pc.h"

/* Global declarations */
extern char* level; /* Pointer to the level file in memory */
extern unsigned levelsize; /* Length of the level file in memory */
extern char endian; /* Endianness. 0=Big-Endian, 1=Little-Endian */

/* Global static declarations */
static char filetype=0; /* Type of TR1 level file. 0=PHD, 1=TUB */
static unsigned short numRooms; /* Number of Rooms */
static int *roomX; /* Array of position of rooms on X axis */
static int *roomY; /* Array of position of rooms on Y axis */
static int *roomZ; /* Array of position of rooms on Z axis */

/* 
 * Main function of tr1pc.
 * This function is called from main.c.
 */
void tr1pc_main(int argc, char *argv[]){
	unsigned freeuint1, freeuint2, freeuint3;
	
	/* Stores whether the level is PHD or TUB */
	filetype=((!strncasecmp(substring(argv[1],strlen(argv[1])-3,3),"tub",3))?1:0);
	
	/* Navigates the level file */
	freeuint1=tr1pc_navigate();
	memcpy(&freeuint2,level+freeuint1,4);
	if (!endian) freeuint2=reverse32(freeuint2);
	freeuint2=freeuint1+4+(freeuint2*22);
	for (freeuint1+=4;freeuint1<freeuint2;freeuint1+=22){
		if ((level[freeuint1]==0)&&(level[freeuint1+1]==0)) break;
	}
	
	/* Checks whether Lara actually exists in this level */
	if (freeuint1==freeuint2){
		printf("ERROR: Lara has not been placed in this level\n");
		exit (-1);
	}
	
	/* Calls the function that moves Lara */
	tr1pc_move_lara(freeuint1,strtol(argv[2],NULL,10),strtol(argv[3],NULL,10),strtol(argv[5],NULL,10),strtol(argv[4],NULL,10),strtol(argv[6],NULL,10));
	
	/* Writes out the level file if necessary */
	writeFile(argv[1],level,levelsize);
}

/*
 * Navigates through the level file, and returns the position of NumItems
 */
unsigned tr1pc_navigate(){
	/* Variable Declarations */
	unsigned curpos; /* Current Position in the file navigation */
	unsigned freeuint1, freeuint2;
	unsigned short freeshort1, freeshort2, freeshort3, freeshort4;
	
	/* Navigates to NumRooms */
	memcpy(&freeuint1,level+4,4);
	if (!endian) freeuint1=reverse32(freeuint1);
	curpos=12+(freeuint1<<16); /* [NumRooms] */
	memcpy(&freeshort1,level+curpos,2);
	if (!endian) freeshort1=reverse16(freeshort1);
	numRooms=freeshort1;
	
	/* Allocates the memory for the arrays of offsets within rooms */
	roomX=malloc(freeshort1*4);
	roomY=malloc(freeshort1*4);
	roomZ=malloc(freeshort1*4);
	
	/* Navigates from NumRooms to the end of the file */
	curpos+=2;
	for (freeshort2=0;freeshort2<freeshort1;freeshort2++){
		memcpy(&roomX[freeshort2], level+curpos, 4);
		if (!endian) roomX[freeshort2]=reverse32(roomX[freeshort2]);
		memcpy(&roomZ[freeshort2], level+curpos+4, 4);
		if (!endian) roomZ[freeshort2]=reverse32(roomZ[freeshort2]);
		memcpy(&roomY[freeshort2], level+curpos+8, 4);
		if (!endian) roomY[freeshort2]=reverse32(roomY[freeshort2]);
		curpos+=20; /* [NumVertices] */
		memcpy(&freeshort3, level+curpos, 2);
		if (!endian) freeshort3=reverse16(freeshort3);
		curpos+=2+(freeshort3<<3); /* [NumRectangles] */
		memcpy(&freeshort3, level+curpos, 2);
		if (!endian) freeshort3=reverse16(freeshort3);
		curpos+=2+(freeshort3*10); /* [NumTriangles] */
		memcpy(&freeshort3, level+curpos, 2);
		if (!endian) freeshort3=reverse16(freeshort3);
		curpos+=2+(freeshort3<<3); /* [NumSprites] */
		memcpy(&freeshort3, level+curpos, 2);
		if (!endian) freeshort3=reverse16(freeshort3);
		curpos+=2+(freeshort3<<2); /* [NumDoors] */
		memcpy(&freeshort3, level+curpos, 2);
		if (!endian) freeshort3=reverse16(freeshort3);
		curpos+=2+(freeshort3<<5); /* [NumZSector] */
		memcpy(&freeshort3, level+curpos, 2);
		if (!endian) freeshort3=reverse16(freeshort3);
		memcpy(&freeshort4, level+curpos+2, 2);
		if (!endian) freeshort4=reverse16(freeshort4);
		curpos+=6+(freeshort4*freeshort3*8); /* [NumLights] */
		memcpy(&freeshort3, level+curpos, 2);
		if (!endian) freeshort3=reverse16(freeshort3);
		curpos+=2+(freeshort3*18); /* [NumStaticMeshes] */
		memcpy(&freeshort3, level+curpos, 2);
		if (!endian) freeshort3=reverse16(freeshort3);
		curpos+=6+(freeshort3*18); /* [AlternateRoom] */
	} /* [NumFloorData] */
	memcpy(&freeuint1, level+curpos, 4);
	if (!endian) freeuint1=reverse32(freeuint1);
	curpos+=4+(freeuint1<<1); /* [NumMeshData] */
	memcpy(&freeuint1, level+curpos, 4);
	if (!endian) freeuint1=reverse32(freeuint1);
	curpos+=4+(freeuint1<<1); /* [NumMeshPointers] */
	memcpy(&freeuint1, level+curpos, 4);
	if (!endian) freeuint1=reverse32(freeuint1);
	curpos+=4+(freeuint1<<2); /* [NumAnimations] */
	memcpy(&freeuint1, level+curpos, 4);
	if (!endian) freeuint1=reverse32(freeuint1);
	curpos+=4+(freeuint1<<5); /* [NumStateChanges] */
	memcpy(&freeuint1, level+curpos, 4);
	if (!endian) freeuint1=reverse32(freeuint1);
	curpos+=4+(freeuint1*6); /* [NumAnimDispatches] */
	memcpy(&freeuint1, level+curpos, 4);
	if (!endian) freeuint1=reverse32(freeuint1);
	curpos+=4+(freeuint1<<3); /* [NumAnimCommands] */
	memcpy(&freeuint1, level+curpos, 4);
	if (!endian) freeuint1=reverse32(freeuint1);
	curpos+=4+(freeuint1<<1); /* [NumMeshTrees] */
	memcpy(&freeuint1, level+curpos, 4);
	if (!endian) freeuint1=reverse32(freeuint1);
	curpos+=4+(freeuint1<<2); /* [NumFrames] */
	memcpy(&freeuint1, level+curpos, 4);
	if (!endian) freeuint1=reverse32(freeuint1);
	curpos+=4+(freeuint1<<1); /* [NumMoveables] */
	memcpy(&freeuint1, level+curpos, 4);
	if (!endian) freeuint1=reverse32(freeuint1);
	curpos+=4+(freeuint1*18); /* [NumStaticMeshes] */
	memcpy(&freeuint1, level+curpos, 4);
	if (!endian) freeuint1=reverse32(freeuint1);
	curpos+=4+(freeuint1<<5); /* [NumObjectTextures] */
	memcpy(&freeuint1, level+curpos, 4);
	if (!endian) freeuint1=reverse32(freeuint1);
	curpos+=4+(freeuint1*20); /* [NumSpriteTextures] */
	memcpy(&freeuint1, level+curpos, 4);
	if (!endian) freeuint1=reverse32(freeuint1);
	curpos+=4+(freeuint1<<4); /* [NumSpriteSequences] */
	memcpy(&freeuint1, level+curpos, 4);
	if (!endian) freeuint1=reverse32(freeuint1);
	curpos+=4+(freeuint1<<3); /* [NumCameras] */
	if (filetype) curpos+=768;
	memcpy(&freeuint1, level+curpos, 4);
	if (!endian) freeuint1=reverse32(freeuint1);
	curpos+=4+(freeuint1<<4); /* [NumSoundSources] */
	memcpy(&freeuint1, level+curpos, 4);
	if (!endian) freeuint1=reverse32(freeuint1);
	curpos+=4+(freeuint1<<4); /* [NumBoxes] */
	memcpy(&freeuint1, level+curpos, 4);
	if (!endian) freeuint1=reverse32(freeuint1);
	curpos+=4+(freeuint1*20); /* [NumOverlaps] */
	memcpy(&freeuint2, level+curpos, 4);
	if (!endian) freeuint2=reverse32(freeuint2);
	curpos+=4+(freeuint2<<1)+(freeuint1*12); /* [NumAnimatedTextures] */
	memcpy(&freeuint1, level+curpos, 4);
	if (!endian) freeuint1=reverse32(freeuint1);
	curpos+=4+(freeuint1<<1); /* [NumItems] */
	return curpos;
}

/*
 * Moves Lara
 * Parameters:
 *   offset=The offset of the item in the level
 *   room=Room
 *   x=X-Position of the item, within the room
 *   y=Y-Position of the item, within the room
 *   z=Z-Position of the item, within the room
 *   angle=Angle of the item, in degrees
 */
void tr1pc_move_lara(unsigned offset, short room, int x, int y, int z, short angle){
	/* Variable Declarations */
	unsigned freeuint1;
	
	/* Checks whether room and item exist */
	if (room>=numRooms){
		printf("ERROR: Room %u does not exist\n",room);
		exit(-1);
	}
	
	/* Adjusts the parameters (where necessary) to fit in the level file */
	x+=roomX[room];
	y=roomY[room]-y;
	z+=roomZ[room];
	angle=(angle/45)<<13;
	if (!endian){
		x=reverse32(x);
		y=reverse32(y);
		z=reverse32(z);
		angle=reverse16(angle);
		room=reverse16(room);
	}
	
	/* Writes the parameters to the level */
	memcpy(level+offset+2,&room,2);
	memcpy(level+offset+4,&x,4);
	memcpy(level+offset+8,&y,4);
	memcpy(level+offset+12,&z,4);
	memcpy(level+offset+16,&angle,2);
}
